/****
 *
 * Exemplo: ListaEncad.c
 *
 * Captulo: 11
 * Seo: 11.8
 *
 * Demonstra: Operaes bsicas sobre listas encadeadas.
 *
 ****/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

typedef  struct  no {
            char        dado[10];
            struct  no  *proximo;
         } tNo, *tLista;

extern void  ImprimeListaEncadeada(tLista  p);
extern void  InsereDepois(tLista  p, char  *stringNovo, char  *stringNaLista);
extern void  RemoveNo(tLista   *p, char   *stringNaLista);

/****
 *
 * ImprimeListaEncadeada()
 *
 * Imprime os ns de uma lista simplesmente encadeada apontada por p.
 *
 ****/
void  ImprimeListaEncadeada(tLista  p)
{
   while  (p)  {
      printf("%s\n", p->dado);    /* Imprime o valor do n e... */
      p = p->proximo;             /* Avana para o prximo n   */
   }
}


/****
 *
 * InsereDepois()
 *
 * Insere um novo n cujo contedo  stringNovo aps o n cujo contedo 
 * stringNaLista na lista apontada por p.
 *
 ****/
void   InsereDepois( tLista  p, char  *stringNovo,
                     char  *stringNaLista )
{
    tLista   posicaoDoNo, noNovo;

    posicaoDoNo = NULL;

    while  (p  &&  !posicaoDoNo){ 	/* Procura o n */
       if   (!strcmp(p->dado, stringNaLista))
	   posicaoDoNo = p;		/* N procurado foi encontrado */
	 else
	   p = p->proximo;		/* Passa para o prximo n */
    }

    if  (posicaoDoNo){ /* posicaoDoNo foi encontrado e aponta para */
			     /* o n aps o qual a insero ser feita   */
	 noNovo = malloc(sizeof(tNo)); /* Aloca o novo n */
	 if (noNovo) {
              /* Preenche o contedo do novo n */
	   strcpy(noNovo->dado, stringNovo);

             /* Faz o novo n apontar para o n   */
             /* apontado antes pelo n encontrado */
	   noNovo->proximo = posicaoDoNo->proximo;

             /* Faz o n encontrado apontar para o n  novo */
	   posicaoDoNo->proximo = noNovo;
	 }  /* if */
    } /* if */
}  /* InsereDepois */


/****
 * RemoveNo()
 *
 * Remove o primeiro n encontrado cujo contedo 
 * stringNaLista na lista apontada por p.
 ****/

void   RemoveNo(tLista   *p, char   *stringNaLista)
{
   tLista   noAnterior = NULL, /* No h n anterior no incio  */
            posicaoDoNo= *p; /* Comea busca no incio da lista */
   unsigned noFoiEncontrado = 0; /* O n no foi ainda encontrado */

   while  (posicaoDoNo  &&  !noFoiEncontrado)  {  /* Procura o n */
      if   (!strcmp(posicaoDoNo->dado, stringNaLista))
         noFoiEncontrado = 1; /* N procurado foi encontrado  */
      else {                       /* O n anterior passa a ser */
         noAnterior = posicaoDoNo; /* o atual e o atual passa   */
         posicaoDoNo = posicaoDoNo->proximo;  /* a ser o prximo */
      }
   }

   if  (noFoiEncontrado) {
      if  (!noAnterior) /* O n a remover  o primeiro da lista */
         *p = (*p)->proximo; /* Primeiro n ser antigo segundo n  */
      else   /* O n a ser removido NO  o primeiro da lista */
         noAnterior->proximo = posicaoDoNo->proximo;/* Faz o desvio */

      /* Neste ponto, o n apontado por posicaoDoNo no mais      */
      /* pertence  lista e a remoo est logicamente concluda. */
      /* Resta apenas liberar o espao ocupado pelo n removido.  */

         /* Libera a alocao de memria do n removido */
       free(posicaoDoNo);
    }
}


int main(void)
{
   tLista   lista, ponteiroAux1, ponteiroAux2;

   lista = malloc(sizeof(tNo));           /* 1 */
   strcpy(lista->dado, "azul");           /* 2 */
   ponteiroAux1 = malloc(sizeof(tNo));    /* 3 */
   strcpy(ponteiroAux1->dado, "branco");  /* 4 */
   lista->proximo = ponteiroAux1;         /* 5 */
   ponteiroAux2 = malloc(sizeof(tNo));    /* 6 */
   strcpy(ponteiroAux2->dado, "verde");   /* 7 */
   ponteiroAux1->proximo = ponteiroAux2;  /* 8 */
   ponteiroAux2->proximo = NULL;          /* 9 */

   ImprimeListaEncadeada(lista);

   InsereDepois(lista, "vermelho", "branco");
   printf("\n\n\n");
   ImprimeListaEncadeada(lista);

   RemoveNo(&lista, "verde");
   printf("\n\n\n");
   ImprimeListaEncadeada(lista);

   return 0;
}

