/****
 *
 * Exemplo: CriptoXor1.c
 *
 * Captulo: 13
 * Seo: 13.7.2
 *
 * Demonstra: Implementao de criptografia xor usando
 *            um caractere como chave.
 *
 ****/

#include <stdio.h>

/****
 *
 * Funo Criptografa():
 *
 * Descrio: Faz criptografia xor de um arquivo.
 *
 * Parmetros:
 *      arquivo (entrada): nome do arquivo que ser criptografado
 *      chave (entrada): caractere que representa a
 *                       chave criptogrfica
 *
 * Retorno: 1, se a operao for bem sucedida e 0 em caso contrrio.
 *
 ****/

unsigned Criptografa(const char *arquivo, char chave)
{
   char c;
   FILE *ptrArquivo, /* Stream associado ao arquivo original */
        *ptrTemp;    /* Stream associado a um arquivo temporrio */

  /* Abre arquivo original para leitura e gravao no modo binrio */
   ptrArquivo = fopen(arquivo, "r+b");

   if (!ptrArquivo)
      return 0;   /* Arquivo no pode ser aberto */

   ptrTemp = tmpfile();  /* Cria arquivo temporrio */

   if (!ptrTemp) {
      fclose(ptrArquivo); /* Arquivo temporrio no foi aberto */
      return 0;
   }

      /* Enquanto o final do arquivo original no for atingido, */
      /* l cada byte neste arquivo, executa xor do byte lido   */
      /* com a chave e grava o resultado no arquivo temporrio  */
   while (1) {
      c = getc(ptrArquivo);
      if (feof(ptrArquivo))   /* Testa se final do arquivo */
         break;               /* de entrada foi atingido   */
      putc(c ^ chave, ptrTemp);
   }

         /* Antes de copiar o contedo criptografado do     */
         /* arquivo temporrio para o arquivo original,     */
         /*  necessrio retornar ao incio de cada arquivo */
   rewind(ptrArquivo);
   rewind(ptrTemp);

   /* Copia contedo do arquivo temporrio para o arquivo original */
   while (1) {
      c = getc(ptrTemp);
      if (feof(ptrTemp))         /* Testa se final do arquivo */
         break;                  /* temporrio foi atingido   */
      putc(c, ptrArquivo);
   }

          /* Fecha arquivos */
   fclose(ptrArquivo);
   fclose(ptrTemp); /* Arquivo temporrio automaticamente destrudo */

   return 1;
}

int main(int argc, char **argv)
{
   unsigned char aChave;

      /* Este programa funciona apenas quando     */
      /* recebe um nome de arquivo como argumento */
   if (argc != 2) {
      printf("Erro: nome de arquivo ausente.");
      return 1;
   }

   printf("\nIntroduza a chave: ");
   aChave = getchar();

   if ( !Criptografa(argv[1], aChave) ) {
      printf("Erro tentando criptografar arquivo");
      return 1;
   }

   return 0;
}
