/****
 *
 * Exemplo: CriptoXor2.c
 *
 * Captulo: 13
 * Seo: 13.7.2
 *
 * Demonstra: Implementao de criptografia xor usando
 *            um string como chave.
 *
 ****/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define TAM_ARRANJO  20  /* Tamanho do arranjo que ir conter a chave */

extern void LimpaBuffer(void);
char *LeString(char * buffer, unsigned nElementos);
extern unsigned Criptografa2(const char *arquivo, const char *chave);

int main(int argc, char **argv)
{
   char 	*aChave, buffer[TAM_ARRANJO + 1];
   unsigned operacaoOK;

      /* Este programa funciona apenas quando     */
      /* recebe um nome de arquivo como argumento */
   if (argc != 2) {
      printf("Erro: nome de arquivo ausente.");
      return 1;
   }

   printf("\nIntroduza a chave: ");
   aChave = LeString(buffer, TAM_ARRANJO);
   while (!aChave && !*aChave) {
      printf("\nValor incorreto. Reintroduza a chave: ");
      aChave = LeString(buffer, TAM_ARRANJO);
   }

   printf("\nComprimindo arquivo \"%s\"...", argv[1]);
   operacaoOK = Criptografa2(argv[1], aChave);

   if (!operacaoOK) {
      printf("\nErro tentando criptografar arquivo");
      return 1;
   }
   else
      printf("\nCompressao concluida");

   getchar();
   return 0;
}

/****
 *
 * A funo LeString() l um string no meio de entrada contendo um nmero
 * mximo de caracteres especificado
 *
 * Argumentos: Sada: buffer - arranjo de caracteres que conter o string lido
 *			   Entrada: nElementos - tamanho do arranjo buffer
 * Retorno: ponteiro para o string lido ou NULL, caso contrrio.
 *
 ****/
char *LeString(char * buffer, unsigned nElementos)
{
   unsigned contador = 0;   /* Conta o nmero de caracteres lidos */
   unsigned caracterLido;

   		/* L no mximo nElementos - 1 caracteres ou at encontrar '\n' */
   caracterLido = getchar();
   while ((caracterLido != '\n') && (contador < nElementos-1)) {
      buffer[contador++] = (char) caracterLido;
      caracterLido = getchar();
   }

   buffer[contador] = '\0'; /* Termina o string */

   if ((caracterLido == '\n') || (getchar() == '\n'))  /* Tudo bem */
      return buffer;
   else {      /* Usurio digitou caracteres demais */
      LimpaBuffer(); /* Limpa a sujeira deixada pelo usurio */
      return (char *) 0;
   }
}

/****
 *
 * A funo LimpaBuffer() l e descarta todos os caracteres que porventura
 * tenham sido deixados no buffer de entrada numa tentativa de leitura de dados
 *
 * Argumentos: nenhum
 * Valor de retorno: nenhum
 *
 ****/
void LimpaBuffer(void)
{
   int valorLido = getchar(); /* valorLido deve ser int! */

         /* L caracteres no buffer de entrada enquanto    */
         /* no econtra final de linha ou final de arquivo */
   while ((valorLido != '\n') && (valorLido != EOF))
      valorLido = getchar();
}

/****
 *
 * Funo Criptografa2():
 *
 * Descrio: Faz criptografia xor de um arquivo. A diferena
 *            com relao  funo Criptografa() anterior 
 *            que a funo atual recebe um string como chave,
 *            ao invs de um nico caractere.
 *
 * Parmetros:
 *      arquivo (entrada): nome do arquivo que ser criptografado
 *      chave (entrada): string que representa a chave criptogrfica
 *
 * Retorno: 1, se a operao for bem sucedida e 0 em caso contrrio.
 *
 ****/

unsigned Criptografa2(const char *arquivo, const char *chave)
{
   char      *sequenciaBytes, c;
   size_t    tamanhoChave, nBytesLidos;
   unsigned  i;
   FILE      *ptrArquivo, /* Stream associado ao arquivo original */
             *ptrTemp;   /* Stream associado a um arquivo temporrio */

   /* Abre arquivo original para leitura e gravao no modo binrio */
   ptrArquivo = fopen(arquivo, "r+b");

   if (!ptrArquivo)
      return 0;   /* Arquivo no pode ser aberto */

   ptrTemp = tmpfile();  /* Cria arquivo temporrio */

   if (!ptrTemp) {
      fclose(ptrArquivo); /* Arquivo temporrio no pode ser aberto */
      return 0;
   }

      /* O arquivo ser lido em quantidades de bytes iguais */
      /* ao tamanho da chave. O array sequenciaBytes        */
      /* ser utilizado para esta finalidade.               */
   tamanhoChave = strlen(chave);
   sequenciaBytes = malloc(tamanhoChave);

   if (!sequenciaBytes)
      return 0;

     /* Enquanto o final do arquivo original no for atingido, l  */
     /* seqncias de bytes do tamanho da chave neste arquivo, faz */
     /* xor da seqncia lida com a chave, byte a byte e grava no  */
     /* arquivo temporrio.                                        */
   do {
      nBytesLidos = fread( sequenciaBytes, 1,
                           tamanhoChave, ptrArquivo );

      for (i = 0; i < nBytesLidos; i++)
         sequenciaBytes[i] ^= chave[i];

      fwrite(sequenciaBytes, 1, nBytesLidos, ptrTemp);

      if (ferror(ptrTemp))
         break;
   } while ( nBytesLidos == tamanhoChave );

       /* Verifica se houve erro de leitura/escrita. */
       /* Se for o caso, fecha arquivos e retorna.   */
   if (ferror(ptrArquivo) || ferror(ptrTemp)) {
      fclose(ptrArquivo);
      fclose(ptrTemp);
      return 0;
   }

         /*  necessrio voltar ao incio de cada arquivo */
   rewind(ptrArquivo);
   rewind(ptrTemp);

      /* Copia contedo do arquivo temporrio para arquivo original */
   while (1) {
      c = getc(ptrTemp);
      if (feof(ptrTemp))       /* Testa se final do arquivo */
         break;                /* temporrio foi atingido   */
      putc(c, ptrArquivo);
   }

         /* Fecha arquivos */
   fclose(ptrArquivo);
   fclose(ptrTemp);   /* Sistema remove arquivo temporrio  */

   free(sequenciaBytes);  /*  necessrio liberar o espao alocado */

   return 1;
}


