/****
 *
 * Exemplo: Flags.c
 *
 * Captulo: 13
 * Seo: 113.7.1
 *
 * Demonstra: Operaes com sinalizadores.
 *
 ****/

#include <stdio.h>

#define DIREITA  (0x1 << 0) /* Tecla direita ocupa bit 0    */
#define ESQUERDA (0x1 << 1) /* Tecla esquerda ocupa bit 1   */
#define CIMA     (0x1 << 2) /* Tecla para cima ocupa bit 2  */
#define BAIXO    (0x1 << 3) /* Tecla para baixo ocupa bit 3 */

void LigaSinalizador(unsigned *sinalizadores, unsigned mascara)
{
   *sinalizadores |= mascara;
}

void DesligaSinalizador(unsigned *sinalizadores, unsigned mascara)
{
   *sinalizadores &= ~mascara;
}

void InverteSinalizador(unsigned *sinalizadores, unsigned mascara)
{
   *sinalizadores ^= mascara;
}

unsigned TestaSinalizador( unsigned sinalizadores, unsigned mascara )
{
   return sinalizadores & mascara;
}

void ImprimeStatusTeclas(unsigned teclas)
{
   switch(teclas) {
      case DIREITA | ESQUERDA | CIMA | BAIXO:
         printf("Todas as teclas estao ligadas\n");
         break;
      case DIREITA | ESQUERDA | CIMA:
         printf("As teclas direita, esquerda e cima estao ligadas\n");
         break;
      case DIREITA | ESQUERDA | BAIXO:
         printf("As teclas direita, esquerda e baixo estao ligadas\n");
         break;
      case DIREITA | CIMA | BAIXO:
         printf("As teclas direita, cima e baixo estao ligadas\n");
         break;
      case ESQUERDA | CIMA | BAIXO:
         printf("As teclas esquerda, cima e baixo estao ligadas\n");
         break;
      case DIREITA | ESQUERDA:
         printf("As teclas direita e esquerda estao ligadas\n");
         break;
      case DIREITA | CIMA:
         printf("As teclas direita e cima estao ligadas\n");
         break;
      case DIREITA | BAIXO:
         printf("As teclas direita e baixo estao ligadas\n");
         break;
      case ESQUERDA | CIMA:
         printf("As teclas esquerda e cima estao ligadas\n");
         break;
      case ESQUERDA | BAIXO:
         printf("As teclas esquerda e baixo estao ligadas\n");
         break;
      case CIMA | BAIXO:
         printf("As teclas cima e baixo estao ligadas\n");
         break;
      case DIREITA:
         printf("A tecla direita esta' ligada\n");
         break;
      case ESQUERDA:
         printf("A tecla esquerda esta' ligada\n");
         break;
      case CIMA:
         printf("A tecla cima esta' ligada\n");
         break;
      case BAIXO:
         printf("A tecla baixo esta' ligada\n");
         break;
      default:
         printf("Nenhuma tecla esta' ligada\n");
   }
}

int main(void)
{
   unsigned teclas = 0;

   LigaSinalizador(&teclas, ESQUERDA);
   ImprimeStatusTeclas(teclas);

   DesligaSinalizador(&teclas, ESQUERDA);
   ImprimeStatusTeclas(teclas);

       /* Inverte ao mesmo tempo os bits correspondentes s teclas */
       /* CIMA e BAIXO; como eles esto zerados, sero ligados     */
   InverteSinalizador(&teclas, CIMA | BAIXO);
   ImprimeStatusTeclas(teclas);

   if (TestaSinalizador(teclas, CIMA))
      printf("TestaSinalizador(): a tecla cima esta' ligada\n");
   else
      printf("TestaSinalizador(): a tecla cima NAO esta' ligada\n");

       /* Neste ponto, as teclas CIMA e BAIXO esto */
       /* ligadas; a seguir, apenas a tecla CIMA   */
       /* desligada; a tecla BAIXO continua ligada  */
   DesligaSinalizador(&teclas, CIMA);
   ImprimeStatusTeclas(teclas);

       /* A instruo a seguir desliga todas as teclas. */
       /* Evidentemente, isto poderia ser feito de modo */
       /* mais simples com atribuio: teclas = 0;      */
   DesligaSinalizador(&teclas, DIREITA | ESQUERDA | CIMA | BAIXO);
   ImprimeStatusTeclas(teclas);

   return 0;
}
